@@ -94,6 +94,18 @@ static struct equiv_cpu_table {
 	struct equiv_cpu_entry *entry;
 } equiv_table;
 
+union zen_patch_rev {
+	struct {
+		__u32 rev	 : 8,
+		      stepping	 : 4,
+		      model	 : 4,
+		      __reserved : 4,
+		      ext_model	 : 4,
+		      ext_fam	 : 8;
+	};
+	__u32 ucode_rev;
+};
+
 union cpuid_1_eax {
 	struct {
 		__u32 stepping    : 4,
@@ -159,100 +171,36 @@ static int cmp_id(const void *key, const void *elem)
 		return 1;
 }
 
-static u32 cpuid_to_ucode_rev(unsigned int val)
-{
-	union zen_patch_rev p = {};
-	union cpuid_1_eax c;
-
-	c.full = val;
-
-	p.stepping  = c.stepping;
-	p.model     = c.model;
-	p.ext_model = c.ext_model;
-	p.ext_fam   = c.ext_fam;
-
-	return p.ucode_rev;
-}
-
-static u32 get_cutoff_revision(u32 rev)
-{
-	switch (rev >> 8) {
-	case 0x80012: return 0x8001277; break;
-	case 0x80082: return 0x800820f; break;
-	case 0x83010: return 0x830107c; break;
-	case 0x86001: return 0x860010e; break;
-	case 0x86081: return 0x8608108; break;
-	case 0x87010: return 0x8701034; break;
-	case 0x8a000: return 0x8a0000a; break;
-	case 0xa0010: return 0xa00107a; break;
-	case 0xa0011: return 0xa0011da; break;
-	case 0xa0012: return 0xa001243; break;
-	case 0xa0082: return 0xa00820e; break;
-	case 0xa1011: return 0xa101153; break;
-	case 0xa1012: return 0xa10124e; break;
-	case 0xa1081: return 0xa108109; break;
-	case 0xa2010: return 0xa20102f; break;
-	case 0xa2012: return 0xa201212; break;
-	case 0xa4041: return 0xa404109; break;
-	case 0xa5000: return 0xa500013; break;
-	case 0xa6012: return 0xa60120a; break;
-	case 0xa7041: return 0xa704109; break;
-	case 0xa7052: return 0xa705208; break;
-	case 0xa7080: return 0xa708009; break;
-	case 0xa70c0: return 0xa70C009; break;
-	case 0xaa001: return 0xaa00116; break;
-	case 0xaa002: return 0xaa00218; break;
-	case 0xb0021: return 0xb002146; break;
-	case 0xb0081: return 0xb008111; break;
-	case 0xb1010: return 0xb101046; break;
-	case 0xb2040: return 0xb204031; break;
-	case 0xb4040: return 0xb404031; break;
-	case 0xb4041: return 0xb404101; break;
-	case 0xb6000: return 0xb600031; break;
-	case 0xb6080: return 0xb608031; break;
-	case 0xb7000: return 0xb700031; break;
-	default: break;
-
-	}
-	return 0;
-}
-
 static bool need_sha_check(u32 cur_rev)
 {
-	u32 cutoff;
-
-	if (!cur_rev) {
-		cur_rev = cpuid_to_ucode_rev(bsp_cpuid_1_eax);
-		pr_info_once("No current revision, generating the lowest one: 0x%x\n", cur_rev);
+	switch (cur_rev >> 8) {
+	case 0x80012: return cur_rev <= 0x800126f; break;
+	case 0x83010: return cur_rev <= 0x830107c; break;
+	case 0x86001: return cur_rev <= 0x860010e; break;
+	case 0x86081: return cur_rev <= 0x8608108; break;
+	case 0x87010: return cur_rev <= 0x8701034; break;
+	case 0x8a000: return cur_rev <= 0x8a0000a; break;
+	case 0xa0011: return cur_rev <= 0xa0011da; break;
+	case 0xa0012: return cur_rev <= 0xa001243; break;
+	case 0xa1011: return cur_rev <= 0xa101153; break;
+	case 0xa1012: return cur_rev <= 0xa10124e; break;
+	case 0xa1081: return cur_rev <= 0xa108109; break;
+	case 0xa2010: return cur_rev <= 0xa20102f; break;
+	case 0xa2012: return cur_rev <= 0xa201212; break;
+	case 0xa6012: return cur_rev <= 0xa60120a; break;
+	case 0xa7041: return cur_rev <= 0xa704109; break;
+	case 0xa7052: return cur_rev <= 0xa705208; break;
+	case 0xa7080: return cur_rev <= 0xa708009; break;
+	case 0xa70c0: return cur_rev <= 0xa70C009; break;
+	case 0xaa002: return cur_rev <= 0xaa00218; break;
+	default: break;
 	}
 
-	cutoff = get_cutoff_revision(cur_rev);
-	if (cutoff)
-		return cur_rev <= cutoff;
-
 	pr_info("You should not be seeing this. Please send the following couple of lines to x86-<at>-kernel.org\n");
 	pr_info("CPUID(1).EAX: 0x%x, current revision: 0x%x\n", bsp_cpuid_1_eax, cur_rev);
 	return true;
 }
 
-static bool cpu_has_entrysign(void)
-{
-	unsigned int fam   = x86_family(bsp_cpuid_1_eax);
-	unsigned int model = x86_model(bsp_cpuid_1_eax);
-
-	if (fam == 0x17 || fam == 0x19)
-		return true;
-
-	if (fam == 0x1a) {
-		if (model <= 0x2f ||
-		    (0x40 <= model && model <= 0x4f) ||
-		    (0x60 <= model && model <= 0x7f))
-			return true;
-	}
-
-	return false;
-}
-
 static bool verify_sha256_digest(u32 patch_id, u32 cur_rev, const u8 *data, unsigned int len)
 {
 	struct patch_digest *pd = NULL;
@@ -260,7 +208,8 @@ static bool verify_sha256_digest(u32 patch_id, u32 cur_rev, const u8 *data, unsi
 	struct sha256_state s;
 	int i;
 
-	if (!cpu_has_entrysign())
+	if (x86_family(bsp_cpuid_1_eax) < 0x17 ||
+	    x86_family(bsp_cpuid_1_eax) > 0x19)
 		return true;
 
 	if (!need_sha_check(cur_rev))
@@ -479,7 +428,6 @@ static int verify_patch(const u8 *buf, size_t buf_size, u32 *patch_size)
 {
 	u8 family = x86_family(bsp_cpuid_1_eax);
 	struct microcode_header_amd *mc_hdr;
-	u32 cur_rev, cutoff, patch_rev;
 	unsigned int ret;
 	u32 sh_psize;
 	u16 proc_id;
@@ -523,24 +471,6 @@ static int verify_patch(const u8 *buf, size_t buf_size, u32 *patch_size)
 	if (patch_fam != family)
 		return 1;
 
-	cur_rev = get_patch_level();
-
-	/* No cutoff revision means old/unaffected by signing algorithm weakness => matches */
-	cutoff = get_cutoff_revision(cur_rev);
-	if (!cutoff)
-		goto ok;
-
-	patch_rev = mc_hdr->patch_id;
-
-	if (cur_rev <= cutoff && patch_rev <= cutoff)
-		goto ok;
-
-	if (cur_rev > cutoff && patch_rev > cutoff)
-		goto ok;
-
-	return 1;
-ok:
-
 	return 0;
 }
 
@@ -664,7 +594,7 @@ static bool __apply_microcode_amd(struct microcode_amd *mc, u32 *cur_rev,
 	unsigned long p_addr = (unsigned long)&mc->hdr.data_code;
 
 	if (!verify_sha256_digest(mc->hdr.patch_id, *cur_rev, (const u8 *)p_addr, psize))
-		return false;
+		return -1;
 
 	native_wrmsrl(MSR_AMD64_PATCH_LOADER, p_addr);
 
@@ -812,6 +742,8 @@ static struct ucode_patch *cache_find_patch(struct ucode_cpu_info *uci, u16 equi
 	n.equiv_cpu = equiv_cpu;
 	n.patch_id  = uci->cpu_sig.rev;
 
+	WARN_ON_ONCE(!n.patch_id);
+
 	list_for_each_entry(p, &microcode_cache, plist)
 		if (patch_cpus_equivalent(p, &n, false))
 			return p;
@@ -1137,7 +1069,7 @@ static enum ucode_state load_microcode_amd(u8 family, const u8 *data, size_t siz
 	if (ret != UCODE_OK)
 		return ret;
 
-	for_each_node_with_cpus(nid) {
+	for_each_node(nid) {
 		cpu = cpumask_first(cpumask_of_node(nid));
 		c = &cpu_data(cpu);
 
@@ -1156,17 +1088,15 @@ static enum ucode_state load_microcode_amd(u8 family, const u8 *data, size_t siz
 
 static int __init save_microcode_in_initrd(void)
 {
+	unsigned int cpuid_1_eax = native_cpuid_eax(1);
 	struct cpuinfo_x86 *c = &boot_cpu_data;
 	struct cont_desc desc = { 0 };
-	unsigned int cpuid_1_eax;
 	enum ucode_state ret;
 	struct cpio_data cp;
 
-	if (microcode_loader_disabled() || c->x86_vendor != X86_VENDOR_AMD || c->x86 < 0x10)
+	if (dis_ucode_ldr || c->x86_vendor != X86_VENDOR_AMD || c->x86 < 0x10)
 		return 0;
 
-	cpuid_1_eax = native_cpuid_eax(1);
-
 	if (!find_blobs_in_containers(&cp))
 		return -EINVAL;
 
